# -- coding: utf-8 --
"""Contém todas as estruturas referentes a manipulação dos dados dentro e fora
do banco de dados.
"""

import psycopg2
from settings import *
from datetime import date


class SingletonDB(object):
    meta = {}
    top_donator_ids = []

    # Singleton class implementation
    _instance = None
    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(SingletonDB, cls).__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self):
        # Definition Variables
        self.dbname = DBNAME
        self.dbuser = DBUSER
        self.dbpassword = DBPASSWORD
        # Database connection
        self.conn = psycopg2.connect(database=self.dbname, user=self.dbuser,
                        password=self.dbpassword)
        self.cur = self.conn.cursor()
        # Extraction settings
        self.min_donations = EXT_MIN_DONATIONS
        self.donators_limit = EXT_DONATORS_LIMIT
        # Metadata initialization
        q = "SELECT ano, mes, dia FROM dim_data ORDER BY ano LIMIT 1"
        rows = self.query_exec(q)
        self.meta['start_date'] = date(rows[0][0], rows[0][1], rows[0][2])
        q = "SELECT ano, mes, dia FROM dim_data ORDER BY ano DESC LIMIT 1"
        rows = self.query_exec(q)
        self.meta['end_date'] = date(rows[0][0], rows[0][1], rows[0][2])
        self.top_donator_ids = self._top_donator_ids()
        self.meta['num_donators'] = len(self.top_donator_ids)


    def query_exec(self, q):
        """Executa uma query e devolve a tabela resposta"""
        self.cur.execute(q)
        rows = self.cur.fetchall()
        return rows

    def _top_donator_ids(self):
        """Devolve uma lista com todos os ids de doadores por ordem de maior
        número de doações efetuadas"""

        # q = "SELECT id_doador, count(id_doador) FROM fato_doacao GROUP BY id_doador ORDER BY count(*) DESC LIMIT 10"

        q = "SELECT id_doador FROM top_doadores"
        if self.min_donations > 0:
            q += " WHERE num_doacoes >= %d" % self.min_donations
        if self.donators_limit > 0:
            q += " LIMIT %d"  % self.donators_limit
        rows = self.query_exec(q)
        doador_ids = []
        for row in rows:
            doador_ids.append(row[0])

        return doador_ids

    def donations_by_donator(self, id_doador):
        """Devolve uma tabela com todas as doações feitas por um doador
        ordenadas crescentemente pela data.
        
        rows:
            [0]: f.id_doador
            [1]: f.id_triagem
            [2]: f.id_data
            [3]: d.ano
            [4]: d.mes
            [5]: d.dia
            [6]: f.hematocrit
            [7]: f.hemoglobin
        """

        pdict = {'id_doador': id_doador}
        q = "SELECT f.id_doador, f.id_triagem, f.id_data, d.ano, d.mes, d.dia, f.hematocrit, f.hemoglobin FROM fato_doacao f, dim_data d WHERE f.id_data = d.id_data AND f.id_doador = %(id_doador)d ORDER BY d.ano, d.mes, d.dia" % pdict
        rows = self.query_exec(q)
        return rows


class Donator():
    str_template = "{id} {donations_list}\n"

    def __init__ (self, doador_id):
        self.id = doador_id
        self.donations = []

    def __str__ (self):
        kw = {
            'id': self.id,
            'donations_num': len(self.donations),
            'donations_list': self.donations
        }
        return self.str_template.format(**kw)

    def extract_time_series (self):
        db = SingletonDB()

        if not self.donations:
            for row in db.donations_by_donator(self.id):
                data = date(row[3], row[4], row[5])
                data_offset = (data - db.meta['start_date']).days
                hemat = row[6]
                hemog = float(row[7])
                d = (data_offset, hemat, hemog)
                self.donations.append(d)

    def set_time_series (self, donations):
        self.donations = donations
